//
// ping_receiver.cc : Ping Receiver Main File
// author           : Fabio Silva
//
// Copyright (C) 2000-2001 by the Unversity of Southern California
// $Id: ping_receiver.cc,v 1.1 2001/11/21 19:51:23 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include "ping.hh"

void PingApp::recv(NRAttrVec *data, NR::handle my_handle)
{
  NRSimpleAttribute<int> *counterAttr = NULL;
  NRSimpleAttribute<void *> *timeAttr = NULL;
  EventTime *probe_event;
  long delay_seconds;
  long delay_useconds;
  float total_delay;
  struct timeval tmv;

  getTime(&tmv);

  counterAttr = AppCounterAttr.find(data);
  timeAttr = TimeAttr.find(data);

  if (!counterAttr || !timeAttr){
    fprintf(stderr, "Received a BAD packet !\n");
    fflush(NULL);
    printAttrs(data);
    return;
  }

  // Calculate latency
  probe_event = (EventTime *) timeAttr->getVal();
  delay_seconds = tmv.tv_sec;
  delay_useconds = tmv.tv_usec;

  if ((delay_seconds < probe_event->seconds) ||
      ((delay_seconds == probe_event->seconds) &&
       (delay_useconds < probe_event->useconds))){
    // Time's not synchronized
    delay_seconds = -1;
    delay_useconds = 0;
    fprintf(stderr, "Error calculating delay !\n");
    fflush(NULL);
  }
  else{
    delay_seconds = delay_seconds - probe_event->seconds;
    if (delay_useconds < probe_event->useconds){
      delay_seconds--;
      delay_useconds = delay_useconds + 1000000;
    }
    delay_useconds = delay_useconds - probe_event->useconds;
  }
  total_delay = (float) (1.0 * delay_seconds) + ((float) delay_useconds / 1000000.0);

  // Check if this is the first message received
  if (first_msg_recv < 0){
    first_msg_recv = counterAttr->getVal();
  }

  // Print output message
  if (last_seq_recv >= 0){
    if (counterAttr->getVal() < last_seq_recv){
      // Multiple sources detected, disabling statistics
      last_seq_recv = -1;
      fprintf(stderr, "Received data %d, total latency = %f!\n",
	      counterAttr->getVal(), total_delay);
      fflush(NULL);
    }
    else{
      last_seq_recv = counterAttr->getVal();
      num_msg_recv++;
      fprintf(stderr, "Received data: %d, total latency = %f, %% messages received: %f !\n",
	      last_seq_recv, total_delay,
	      (float) ((num_msg_recv * 100.00) /
		       ((last_seq_recv - first_msg_recv) + 1)));
      fflush(NULL);
    }
  }
  else{
    fprintf(stderr, "Received data %d, total latency = %f !\n",
	    counterAttr->getVal(), total_delay);
    fflush(NULL);
  }
}

handle PingApp::setupSubscription()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::INTEREST_CLASS));
  attrs.push_back(LatitudeAttr.make(NRAttribute::GT, 54.78));
  attrs.push_back(LongitudeAttr.make(NRAttribute::LE, 87.32));
  attrs.push_back(TargetAttr.make(NRAttribute::IS, "F117A"));

  handle h = dr->subscribe(&attrs, mr);

  ClearAttrs(&attrs);

  return h;
}

void PingApp::run(int argc, char **argv)
{
  // Parse commnad line
  ParseLine(argc, argv, &diffusion_port);

  // Create diffusion library and receive functions
  dr = NR::createNR(diffusion_port);
  mr = new MyReceive;
  mr->app = this;

  subHandle = setupSubscription();

  // Do nothing
  while (1){
    sleep(100);
  }
}

int main(int argc, char **argv)
{
  PingApp *app;

  app = new PingApp();
  app->run(argc, argv);

  return 0;
}
