//
// ping_sender.cc : Ping Server Main File
// author         : Fabio Silva
//
// Copyright (C) 2000-2001 by the Unversity of Southern California
// $Id: ping_sender.cc,v 1.1 2001/11/21 19:51:24 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include "ping.hh"
#include <unistd.h>

void PingApp::recv(NRAttrVec *data, NR::handle my_handle)
{
  NRSimpleAttribute<int> *nrclass = NULL;

  nrclass = NRClassAttr.find(data);

  if (!nrclass){
    fprintf(stderr, "Received a BAD packet !\n");
    return;
  }

  switch (nrclass->getVal()){

  case NRAttribute::INTEREST_CLASS:

    fprintf(stderr, "Received an Interest message !\n");
    break;

  case NRAttribute::DISINTEREST_CLASS:

    fprintf(stderr, "Received a Disinterest message !\n");
    break;

  default:

    fprintf(stderr, "Received an unknown message (%d)!\n", nrclass->getVal());
    break;

  }
}

handle PingApp::setupSubscription()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::NE, NRAttribute::DATA_CLASS));
  attrs.push_back(NRScopeAttr.make(NRAttribute::IS, NRAttribute::NODE_LOCAL_SCOPE));
  attrs.push_back(TargetAttr.make(NRAttribute::EQ, "F117A"));
  attrs.push_back(LatitudeAttr.make(NRAttribute::IS, 60.00));
  attrs.push_back(LongitudeAttr.make(NRAttribute::IS, 54.00));

  handle h = dr->subscribe(&attrs, mr);

  ClearAttrs(&attrs);

  return h;
}

handle PingApp::setupPublication()
{
  NRAttrVec attrs;

  attrs.push_back(NRClassAttr.make(NRAttribute::IS, NRAttribute::DATA_CLASS));
  attrs.push_back(LatitudeAttr.make(NRAttribute::IS, 60.00));
  attrs.push_back(LongitudeAttr.make(NRAttribute::IS, 54.00));
  attrs.push_back(TargetAttr.make(NRAttribute::IS, "F117A"));

  handle h = dr->publish(&attrs);

  ClearAttrs(&attrs);

  return h;
}

void PingApp::run(int argc, char **argv)
{
  int retval;
  NRAttrVec data_attr;
  NRSimpleAttribute<void *> *timeAttr;
  NRSimpleAttribute<int> *counterAttr;
  struct timeval tmv;
  EventTime *lastEventTime;
  char input;

  // Parse command line
  ParseLine(argc, argv, &diffusion_port);

  dr = NR::createNR(diffusion_port);
  mr = new MyReceive;
  mr->app = this;

  // Setup publication and subscription
  subHandle = setupSubscription();
  pubHandle = setupPublication();

  // Create time attribute
  getTime(&tmv);
  lastEventTime = new EventTime;
  lastEventTime->seconds = tmv.tv_sec;
  lastEventTime->useconds = tmv.tv_usec;
  timeAttr = TimeAttr.make(NRAttribute::IS, (void *) &lastEventTime,
			   sizeof(EventTime));
  data_attr.push_back(timeAttr);

  // Change pointer to point to attribute's payload
  delete lastEventTime;
  lastEventTime = (EventTime *) timeAttr->getVal();

  // Create counter attribute
  counterAttr = AppCounterAttr.make(NRAttribute::IS, last_seq_sent);
  data_attr.push_back(counterAttr);

  // Main thread will send ping probes
  while(1){
#ifdef INTERACTIVE
    fprintf(stdout, "Press <Enter> to send a ping probe...");
    input = getc(stdin);
    if (input == 'c'){
      dr->unsubscribe(subHandle);
      exit(0);
    }
#else
    sleep(5);
#endif

    // Update time in the packet
    getTime(&tmv);
    lastEventTime->seconds = tmv.tv_sec;
    lastEventTime->useconds = tmv.tv_usec;

    // Send data probe
    fprintf(stderr, "Sending Data %d\n", last_seq_sent);
    retval = dr->send(pubHandle, &data_attr);

    // Update counter
    last_seq_sent++;
    counterAttr->setVal(last_seq_sent);
  }
}

int main(int argc, char **argv)
{
  PingApp *app;

  app = new PingApp();
  app->run(argc, argv);

  return 0;
}
